@@ -61,6 +61,8 @@ module Text.Pandoc.Parsing ( module Text.Pandoc.Sources,
                              lineBlockLines,
                              tableWith,
                              tableWith',
+                             tableWithSpans,
+                             tableWithSpans',
                              widthsFromIndices,
                              gridTableWith,
                              gridTableWith',
@@ -68,6 +70,10 @@ module Text.Pandoc.Parsing ( module Text.Pandoc.Sources,
                              TableNormalization (..),
                              toTableComponents,
                              toTableComponents',
+                             toTableComponentsWithSpans,
+                             toTableComponentsWithSpans',
+                             singleRowSpans,
+                             singleColumnSpans,
                              readWith,
                              readWithM,
                              testStringWith,
@@ -303,8 +309,14 @@ import Text.Pandoc.Parsing.GridTable
       gridTableWith',
       tableWith,
       tableWith',
+      tableWithSpans,
+      tableWithSpans',
       toTableComponents,
       toTableComponents',
+      toTableComponentsWithSpans,
+      toTableComponentsWithSpans',
+      singleRowSpans,
+      singleColumnSpans,
       widthsFromIndices,
       TableComponents(..),
       TableNormalization(..) )
