@@ -46,7 +46,7 @@ import Text.Pandoc.Builder
 import Text.Pandoc.Class.PandocMonad (PandocMonad, report)
 import Text.Pandoc.Options
 import Text.Pandoc.Logging (LogMessage(..))
-import Text.Pandoc.Shared (safeRead, extractSpaces)
+import Text.Pandoc.Shared (safeRead, addPandocAttributes)
 import Text.Pandoc.Sources (ToSources(..), sourcesToText)
 import Text.Pandoc.Transforms (headerShift)
 import Text.TeXMath (readMathML, writeTeX)
@@ -173,7 +173,7 @@ List of all DocBook tags, with [x] indicating implemented,
 [ ] errorname - An error name
 [ ] errortext - An error message.
 [ ] errortype - The classification of an error message
-[ ] example - A formal example, with a title
+[x] example - A formal example, with a title
 [ ] exceptionname - The name of an exception
 [ ] fax - A fax number
 [ ] fieldsynopsis - The name of a field in a class definition
@@ -438,7 +438,7 @@ List of all DocBook tags, with [x] indicating implemented,
 [ ] shortaffil - A brief description of an affiliation
 [ ] shortcut - A key combination for an action that is also accessible through
     a menu
-[ ] sidebar - A portion of a document that is isolated from the main
+[x] sidebar - A portion of a document that is isolated from the main
     narrative flow
 [ ] sidebarinfo - Meta-information for a Sidebar
 [x] simpara - A paragraph that contains only text and inline markup, no block
@@ -548,6 +548,7 @@ data DBState = DBState{ dbSectionLevel  :: Int
                       , dbBook          :: Bool
                       , dbContent       :: [Content]
                       , dbLiteralLayout :: Bool
+                      , dbElementStack  :: [Text]
                       } deriving Show
 
 instance Default DBState where
@@ -556,7 +557,9 @@ instance Default DBState where
                , dbMeta = mempty
                , dbBook = False
                , dbContent = []
-               , dbLiteralLayout = False }
+               , dbLiteralLayout = False
+               , dbElementStack = []
+               }
 
 
 readDocBook :: (PandocMonad m, ToSources a)
@@ -621,9 +624,13 @@ named s e = qName (elName e) == s
 --
 
 addMetadataFromElement :: PandocMonad m => Element -> DB m Blocks
-addMetadataFromElement e =
-  mempty <$ mapM_ handleMetadataElement
+addMetadataFromElement e = do
+  -- Add metadata if at root or appropriate parent element
+  elementStack <- gets dbElementStack
+  if take 1 elementStack `elem` [[], ["book"], ["article"]]
+    then mempty <$ mapM_ handleMetadataElement
                   (filterChildren ((isMetadataField . qName . elName)) e)
+    else return mempty
  where
   handleMetadataElement elt =
     case qName (elName elt) of
@@ -724,9 +731,7 @@ blockTags = Set.fromList $
   , "articleinfo"
   , "attribution"
   , "authorinitials"
-  , "bibliodiv"
   , "biblioentry"
-  , "bibliography"
   , "bibliomisc"
   , "bibliomixed"
   , "blockquote"
@@ -765,31 +770,19 @@ blockTags = Set.fromList $
   , "preface"
   , "procedure"
   , "programlisting"
-  , "qandadiv"
   , "question"
-  , "refsect1"
   , "refsect1info"
-  , "refsect2"
   , "refsect2info"
-  , "refsect3"
   , "refsect3info"
-  , "refsection"
   , "refsectioninfo"
   , "screen"
-  , "sect1"
   , "sect1info"
-  , "sect2"
   , "sect2info"
-  , "sect3"
   , "sect3info"
-  , "sect4"
   , "sect4info"
-  , "sect5"
   , "sect5info"
-  , "section"
   , "sectioninfo"
   , "simpara"
-  , "simplesect"
   , "substeps"
   , "subtitle"
   , "table"
@@ -797,7 +790,13 @@ blockTags = Set.fromList $
   , "titleabbrev"
   , "toc"
   , "variablelist"
-  ] ++ admonitionTags
+  ] ++ sectionTags ++ admonitionTags
+
+sectionTags :: [Text]
+sectionTags = ["bibliography", "bibliodiv"
+              , "sect1", "sect2", "sect3", "sect4", "sect5", "section", "simplesect"
+              , "refsect1", "refsect2", "refsect3", "refsection", "qandadiv"
+              ]
 
 admonitionTags :: [Text]
 admonitionTags = ["caution","danger","important","note","tip","warning"]
@@ -853,9 +852,16 @@ getMediaobject e = do
   fmap (imageWith attr imageUrl tit) capt
 
 getBlocks :: PandocMonad m => Element -> DB m Blocks
-getBlocks e =  mconcat <$>
-                 mapM parseBlock (elContent e)
+getBlocks e = do
+  modify (\st -> st{ dbElementStack = qName (elName e) : dbElementStack st })
+  blocks <- mconcat <$> mapM parseBlock (elContent e)
+  modify (\st -> st{ dbElementStack = drop 1 $ dbElementStack st })
+  return blocks
 
+getRoleAttr :: Element -> [(Text, Text)] -- extract role attribute and add it to the attribute list
+getRoleAttr e = case attrValue "role" e of
+                  "" -> []
+                  r  -> [("role", r)]
 
 parseBlock :: PandocMonad m => Content -> DB m Blocks
 parseBlock (Text (CData CDataRaw _ _)) = return mempty -- DOCTYPE
@@ -863,8 +869,8 @@ parseBlock (Text (CData _ s _)) = if T.all isSpace s
                                      then return mempty
                                      else return $ plain $ trimInlines $ text s
 parseBlock (CRef x) = return $ plain $ str $ T.toUpper x
-parseBlock (Elem e) =
-  case qName (elName e) of
+parseBlock (Elem e) = do
+  parsedBlock <- case qName (elName e) of
         "toc"   -> skip -- skip TOC, since in pandoc it's autogenerated
         "index" -> skip -- skip index, since page numbers meaningless
         "para"  -> parseMixed para (elContent e)
@@ -913,8 +919,8 @@ parseBlock (Elem e) =
         "refsect2" -> sect 2
         "refsect3" -> sect 3
         "refsection" -> gets dbSectionLevel >>= sect . (+1)
-        l | l `elem` titledBlockElements -> parseAdmonition l
-        l | l `elem` admonitionTags -> parseAdmonition l
+        l | l `elem` titledBlockElements -> parseAdmonition False l
+        l | l `elem` admonitionTags -> parseAdmonition True l
         "area" -> skip
         "areaset" -> skip
         "areaspec" -> skip
@@ -976,6 +982,9 @@ parseBlock (Elem e) =
         "title" -> return mempty     -- handled in parent element
         "subtitle" -> return mempty  -- handled in parent element
         _ -> skip >> getBlocks e
+  if qName (elName e) `elem` sectionTags
+     then return parsedBlock
+     else return $ addPandocAttributes (getRoleAttr e) parsedBlock
    where skip = do
            let qn = qName $ elName e
            let name = if "pi-" `T.isPrefixOf` qn
@@ -1111,7 +1120,10 @@ parseBlock (Elem e) =
            modify $ \st -> st{ dbSectionLevel = n }
            b <- getBlocks e
            modify $ \st -> st{ dbSectionLevel = n - 1 }
-           return $ headerWith (elId, classes, maybeToList titleabbrevElAsAttr++attrs) n' headerText <> b
+           let hdr = addPandocAttributes (getRoleAttr e)
+                        $ headerWith (elId, classes, maybeToList titleabbrevElAsAttr ++ attrs)
+                                      n' headerText
+           return $ hdr <> b
          titleabbrevElAsAttr =
            case filterChild (named "titleabbrev") e `mplus`
                 (filterChild (named "info") e >>=
@@ -1134,17 +1146,20 @@ parseBlock (Elem e) =
            b <- p
            case mbt of
              Nothing -> return b
-             Just t -> return $ divWith (attrValue "id" e,[],[])
+             Just t -> return $ divWith (attrValue "id" e, [], getRoleAttr e)
                          (divWith ("", ["title"], []) (plain t) <> b)
 
          -- Admonitions are parsed into a div. Following other Docbook tools that output HTML,
          -- we parse the optional title as a div with the @title@ class, and give the
          -- block itself a class corresponding to the admonition name.
-         parseAdmonition label = do
+         parseAdmonition alwaysIncludeTitle label = do
            mbt <- getTitle
            -- this will ignore the title element if it is present:
            b <- getBlocks e
-           let t = divWith ("", ["title"], []) (plain $ fromMaybe mempty mbt)
+           let t = maybe mempty (divWith ("", ["title"], []) . plain)
+                        (case mbt of
+                           Nothing | alwaysIncludeTitle -> Just mempty
+                           _ -> mbt)
            -- we also attach the label as a class, so it can be styled properly
            return $ divWith (attrValue "id" e,[label],[]) (t <> b)
 
@@ -1234,8 +1249,8 @@ parseInline (Text (CData _ s _)) = do
      else return $ text s
 parseInline (CRef ref) =
   return $ text $ fromMaybe (T.toUpper ref) $ lookupEntity ref
-parseInline (Elem e) =
-  case qName (elName e) of
+parseInline (Elem e) = do
+  parsedInline <- case qName (elName e) of
         "anchor" -> do
            return $ spanWith (attrValue "id" e, [], []) mempty
         "phrase" -> do
@@ -1357,6 +1372,9 @@ parseInline (Elem e) =
         -- <?asciidor-br?> to in handleInstructions, above.
         "pi-asciidoc-br" -> return linebreak
         _          -> skip >> innerInlines id
+  return $ case qName (elName e) of
+    "emphasis" -> parsedInline
+    _ -> addPandocAttributes (getRoleAttr e) parsedInline
    where skip = do
            let qn = qName $ elName e
            let name = if "pi-" `T.isPrefixOf` qn
@@ -1365,8 +1383,7 @@ parseInline (Elem e) =
            lift $ report $ IgnoredElement name
            return mempty
 
-         innerInlines f = extractSpaces f . mconcat <$>
-                          mapM parseInline (elContent e)
+         innerInlines f = f . mconcat <$> mapM parseInline (elContent e)
          codeWithLang = do
            let classes' = case attrValue "language" e of
                                "" -> []
