@@ -14,7 +14,7 @@ module Text.Pandoc.Readers.LaTeX.Math
   )
 where
 import Data.Maybe (fromMaybe, mapMaybe, listToMaybe)
-import Data.List (foldl')
+import qualified Data.List as L
 import Text.Pandoc.Walk (walk)
 import Text.Pandoc.Builder as B
 import qualified Data.Sequence as Seq
@@ -160,7 +160,7 @@ newtheorem inline = do
 extractLabelFromBlock :: Block -> Maybe Text
 extractLabelFromBlock (Para inlines) = extractLabel Nothing inlines
   where
-    extractLabel = foldl' go
+    extractLabel = L.foldl' go
     go :: Maybe Text -> Inline -> Maybe Text
     go (Just t) _ = Just t
     go Nothing (Span (_, _, attrs) _) = lookup "label" attrs
