@@ -24,7 +24,8 @@ import Control.Monad
 import Control.Monad.Except (throwError)
 import qualified Data.Bifunctor as Bifunctor
 import Data.Char (isAlphaNum, isPunctuation, isSpace)
-import Data.List (transpose, elemIndex, sortOn, foldl')
+import Data.List (transpose, elemIndex, sortOn)
+import qualified Data.List as L
 import qualified Data.Map as M
 import Data.Maybe
 import qualified Data.Set as Set
@@ -388,7 +389,7 @@ referenceKey = try $ do
   addKvs <- option [] $ guardEnabled Ext_mmd_link_attributes
                           >> many (try $ spnl >> keyValAttr)
   blanklines
-  let attr'  = extractIdClass $ foldl' (\x f -> f x) attr addKvs
+  let attr'  = extractIdClass $ L.foldl' (\x f -> f x) attr addKvs
       target = (escapeURI $ trimr src, tit)
   st <- getState
   let oldkeys = stateKeys st
@@ -648,7 +649,7 @@ attributes = try $ do
   spnl
   attrs <- many (attribute <* spnl)
   char '}'
-  return $ foldl' (\x f -> f x) nullAttr attrs
+  return $ L.foldl' (\x f -> f x) nullAttr attrs
 
 attribute :: PandocMonad m => MarkdownParser m (Attr -> Attr)
 attribute = identifierAttr <|> classAttr <|> keyValAttr <|> specialAttr
@@ -817,9 +818,28 @@ emailBlockQuote = try $ do
 blockQuote :: PandocMonad m => MarkdownParser m (F Blocks)
 blockQuote = do
   raw <- emailBlockQuote
+  (mbAlert, raw') <-
+    (do guardEnabled Ext_alerts
+        case raw of
+          (t:ts) | "[!" `T.isPrefixOf` t ->
+              case T.strip t of
+                "[!TIP]" -> pure (Just "tip", ts)
+                "[!WARNING]" -> pure (Just "warning", ts)
+                "[!IMPORTANT]" -> pure (Just "important", ts)
+                "[!CAUTION]" -> pure (Just "caution", ts)
+                "[!NOTE]" -> pure (Just "note", ts)
+                _ -> pure (Nothing, raw)
+          _ -> pure (Nothing, raw))
+      <|> pure (Nothing, raw)
   -- parse the extracted block, which may contain various block elements:
-  contents <- parseFromString' parseBlocks $ T.intercalate "\n" raw <> "\n\n"
-  return $ B.blockQuote <$> contents
+  contents <- parseFromString' parseBlocks $ T.intercalate "\n" raw' <> "\n\n"
+  return $
+    case mbAlert of
+      Nothing -> B.blockQuote <$> contents
+      Just alert ->
+        (B.divWith ("", [alert], [])
+          . (B.divWith ("", ["title"], []) (B.para (B.str (T.toTitle alert))) <>))
+           <$> contents
 
 --
 -- list blocks
@@ -2072,7 +2092,8 @@ inlineNote = do
     updateState $ \st -> st{ stateInNote = True
                            , stateNoteNumber = stateNoteNumber st + 1 }
     contents <- inBalancedBrackets inlines
-    notFollowedBy (char '(' <|> char '[') -- ^[link](foo)^ is superscript
+    notFollowedBy (char '(' <|> char '[' <|> ('{' <$ attributes))
+      -- ^[link](foo)^ is superscript
     updateState $ \st -> st{ stateInNote = False }
     return $ B.note . B.para <$> contents
 
