@@ -38,7 +38,8 @@ module Text.Pandoc.Readers.Textile ( readTextile) where
 import Control.Monad (guard, liftM)
 import Control.Monad.Except (throwError)
 import Data.Char (digitToInt, isUpper)
-import Data.List (intersperse, transpose, foldl')
+import Data.List (intersperse, transpose)
+import qualified Data.List as L
 import Data.List.NonEmpty (NonEmpty(..), nonEmpty)
 import qualified Data.Map as M
 import Data.Text (Text)
@@ -732,7 +733,7 @@ orderedListStartAttr = do
 
 -- | Html / CSS attributes
 attributes :: PandocMonad m => TextileParser m Attr
-attributes = foldl' (flip ($)) ("",[],[]) <$>
+attributes = L.foldl' (flip ($)) ("",[],[]) <$>
   try (do special <- option id specialAttribute
           attrs <- many attribute
           return (special : attrs))
