@@ -56,6 +56,7 @@ module Text.Pandoc.Shared (
                      inlineListToIdentifier,
                      textToIdentifier,
                      isHeaderBlock,
+                     hasLineBreaks,
                      onlySimpleTableCells,
                      isTightList,
                      taskListItemFromAscii,
@@ -90,7 +91,8 @@ import Data.Containers.ListUtils (nubOrd)
 import Data.Char (isAlpha, isLower, isSpace, isUpper, toLower, isAlphaNum,
                   generalCategory, GeneralCategory(NonSpacingMark,
                   SpacingCombiningMark, EnclosingMark, ConnectorPunctuation))
-import Data.List (find, foldl', groupBy, intercalate, intersperse, union)
+import Data.List (find, groupBy, intercalate, intersperse, union)
+import qualified Data.List as L
 import qualified Data.Map as M
 import Data.Maybe (mapMaybe)
 import Data.Monoid (Any (..) )
@@ -547,9 +549,14 @@ makeSectionsWithOffsets numoffsets numbering mbBaseLevel bs =
                         ("number", T.intercalate "." (map tshow newnum)) : kvs
                   _ -> kvs
     let divattr = (ident, "section":classes, kvs')
-    let attr = ("",classes,kvs')
+    let isHeadingAttr ("epub:type",_) = False
+        isHeadingAttr ("role",v) =
+          v `elem` ["tab", "presentation", "none", "treeitem",
+                    "menuitem", "button", "heading"]
+        isHeadingAttr _ = True
+    let hattr = ("",classes, filter isHeadingAttr kvs')
     return $
-      Div divattr (Header level' attr title' : sectionContents') : rest'
+      Div divattr (Header level' hattr title' : sectionContents') : rest'
   go (Div divattr@(dident,dclasses,_) (Header level hattr title':ys) : xs)
       | all (\case
                Header level' _ _ -> level' > level
@@ -602,6 +609,18 @@ uniqueIdent exts title' usedIdents =
                      x  -> x
     numIdent n = baseIdent <> "-" <> tshow n
 
+-- | True if inlines include a LineBreak (possibly embedded), with the exception
+-- of line breaks in Notes.
+hasLineBreaks :: [Inline] -> Bool
+hasLineBreaks = getAny . query isLineBreak . walk removeNote
+  where
+    removeNote :: Inline -> Inline
+    removeNote (Note _) = Str ""
+    removeNote x        = x
+    isLineBreak :: Inline -> Any
+    isLineBreak LineBreak = Any True
+    isLineBreak _         = Any False
+
 -- | True if block is a Header block.
 isHeaderBlock :: Block -> Bool
 isHeaderBlock Header{} = True
@@ -728,7 +747,7 @@ inDirectory path action = E.bracket
 -- | Canonicalizes a file path by removing redundant @.@ and @..@.
 makeCanonical :: FilePath -> FilePath
 makeCanonical = Posix.joinPath . transformPathParts . splitDirectories
- where  transformPathParts = reverse . foldl' go []
+ where  transformPathParts = reverse . L.foldl' go []
         go as        "."  = as
         go ("..":as) ".." = ["..", ".."] <> as
         go (_:as)    ".." = as
@@ -744,7 +763,7 @@ makeCanonical = Posix.joinPath . transformPathParts . splitDirectories
 -- > collapseFilePath "parent/foo/.." ==  "parent"
 -- > collapseFilePath "/parent/foo/../../bar" ==  "/bar"
 collapseFilePath :: FilePath -> FilePath
-collapseFilePath = Posix.joinPath . reverse . foldl' go [] . splitDirectories
+collapseFilePath = Posix.joinPath . reverse . L.foldl' go [] . splitDirectories
   where
     go rs "." = rs
     go r@(p:rs) ".." = case p of
