@@ -163,6 +163,7 @@ escapeString context t
   needsEscape '+' = True
   needsEscape '`' = True
   needsEscape '*' = True
+  needsEscape '#' = True
   needsEscape '_' = True
   needsEscape '<' = True
   needsEscape '>' = True
@@ -593,7 +594,9 @@ inlineListToAsciiDoc opts lst = do
                                    _           -> False
        isSpacy Start (Str xs)
          | Just (c, _) <- T.uncons xs = isPunctuation c || isSpace c
-       isSpacy _ _ = True
+       isSpacy End (Link{}) = True
+       isSpacy End (Image{}) = True
+       isSpacy _ _ = False
 
 setIntraword :: PandocMonad m => Bool -> ADW m ()
 setIntraword b = modify $ \st -> st{ intraword = b }
@@ -609,25 +612,27 @@ inlineToAsciiDoc opts (Emph lst) = do
   contents <- inlineListToAsciiDoc opts lst
   isIntraword <- gets intraword
   let marker = if isIntraword then "__" else "_"
-  return $ marker <> contents <> marker
+  return $ delimited marker marker contents
 inlineToAsciiDoc opts (Underline lst) = do
   contents <- inlineListToAsciiDoc opts lst
-  return $ "[.underline]#" <> contents <> "#"
+  return $ delimited "[.underline]#" "#" contents
 inlineToAsciiDoc opts (Strong lst) = do
   contents <- inlineListToAsciiDoc opts lst
   isIntraword <- gets intraword
   let marker = if isIntraword then "**" else "*"
-  return $ marker <> contents <> marker
+  return $ delimited marker marker contents
 inlineToAsciiDoc opts (Strikeout lst) = do
   contents <- inlineListToAsciiDoc opts lst
-  return $ "[line-through]#" <> contents <> "#"
+  return $ delimited "[line-through]#" "#" contents
 inlineToAsciiDoc opts (Superscript lst) = do
   contents <- inlineListToAsciiDoc opts lst
-  return $ "^" <> contents <> "^"
+  return $ delimited "^" "^" contents
 inlineToAsciiDoc opts (Subscript lst) = do
   contents <- inlineListToAsciiDoc opts lst
-  return $ "~" <> contents <> "~"
-inlineToAsciiDoc opts (SmallCaps lst) = inlineListToAsciiDoc opts lst
+  return $ delimited "~" "~" contents
+inlineToAsciiDoc opts (SmallCaps lst) = do
+  contents <- inlineListToAsciiDoc opts lst
+  return $ delimited "[smallcaps]#" "#" contents
 inlineToAsciiDoc opts (Quoted qt lst) = do
   isLegacy <- gets legacy
   contents <- inlineListToAsciiDoc opts lst
