@@ -35,7 +35,8 @@ import Control.Monad.State.Strict
 import Control.Monad ( liftM, when, foldM, unless )
 import Control.Monad.Trans ( MonadTrans(lift) )
 import Data.Char (ord, isSpace, isAscii)
-import Data.List (intercalate, intersperse, partition, delete, (\\), foldl')
+import Data.List (intercalate, intersperse, partition, delete, (\\))
+import qualified Data.List as L
 import Data.List.NonEmpty (NonEmpty((:|)))
 import Data.Containers.ListUtils (nubOrd)
 import Data.Maybe (fromMaybe, isJust, isNothing)
@@ -125,7 +126,7 @@ defaultWriterState = WriterState {stNotes= [],
 strToHtml :: Text -> Html
 strToHtml t
     | T.any isSpecial t =
-       let !x = foldl' go mempty $ T.groupBy samegroup t
+       let !x = L.foldl' go mempty $ T.groupBy samegroup t
         in x
     | otherwise = toHtml t
   where
@@ -436,7 +437,13 @@ pandocToHtml opts (Pandoc meta blocks) = do
                          defField "scrollProgress" True .
                          defField "scrollActivationWidth" ("0" :: Doc Text) .
                          defField "scrollSnap" ("mandatory" :: Doc Text) .
-                         defField "scrollLayout" ("full" :: Doc Text)
+                         defField "scrollLayout" ("full" :: Doc Text) .
+                         (case writerHighlightMethod opts of
+                            IdiomaticHighlighting
+                             | slideVariant == RevealJsSlides ->
+                              defField "highlight-js" True .
+                              defField "highlightjs-theme" ("monokai" :: Doc Text)
+                            _ -> id)
                       else id) .
                   defField "document-css" (isNothing mCss && slideVariant == NoSlides) .
                   defField "quotes" (stQuotes st) .
@@ -674,7 +681,7 @@ tagWithAttributes opts html5 selfClosing tagname attr =
 
 addAttrs :: PandocMonad m
          => WriterOptions -> Attr -> Html -> StateT WriterState m Html
-addAttrs opts attr h = foldl' (!) h <$> attrsToHtml opts attr
+addAttrs opts attr h = L.foldl' (!) h <$> attrsToHtml opts attr
 
 toAttrs :: PandocMonad m
         => [(Text, Text)] -> StateT WriterState m [Attribute]
@@ -794,8 +801,8 @@ blockToHtmlInner opts (Div (ident, "section":dclasses, dkvs)
         if titleSlide
            -- title slides have no content of their own
            then let (as, bs) = break isSec xs
-                in  (breakOnPauses as, bs)
-           else ([], breakOnPauses xs)
+                in  (walk breakOnPauses as, bs)
+           else ([], walk breakOnPauses xs)
   let secttag  = if html5
                     then H5.section
                     else H.div
@@ -940,6 +947,7 @@ blockToHtmlInner _ HorizontalRule = do
   return $ if html5 then H5.hr else H.hr
 blockToHtmlInner opts (CodeBlock (id',classes,keyvals) rawCode) = do
   html5 <- gets stHtml5
+  slideVariant <- gets stSlideVariant
   id'' <- if T.null id'
              then do
                modify $ \st -> st{ stCodeBlockNum = stCodeBlockNum st + 1 }
@@ -957,23 +965,37 @@ blockToHtmlInner opts (CodeBlock (id',classes,keyvals) rawCode) = do
       adjCode  = if tolhs
                     then T.unlines . map ("> " <>) . T.lines $ rawCode
                     else rawCode
-      highlighted = highlight (writerSyntaxMap opts)
-                      (if html5 then formatHtmlBlock else formatHtml4Block)
-                      (id'',classes',keyvals) adjCode
-      hlCode   = case writerHighlightMethod opts of
-                   Skylighting _ -> highlighted
-                   DefaultHighlighting -> highlighted
-                   _ -> Left ""
-  case hlCode of
-         Left msg -> do
-           unless (T.null msg) $
-             report $ CouldNotHighlight msg
-           addAttrs opts (id',classes,keyvals)
-             $ H.pre $ H.code $ toHtml adjCode
-         Right h -> modify (\st -> st{ stHighlighting = True }) >>
-                    -- we set writerIdentifierPrefix to "" since id'' already
-                    -- includes it:
-                    addAttrs opts{writerIdentifierPrefix = ""} (id'',[],keyvals) h
+      isIdiomaticRevealJs = slideVariant == RevealJsSlides &&
+                            writerHighlightMethod opts == IdiomaticHighlighting
+  if isIdiomaticRevealJs
+     then do
+       -- For idiomatic reveal.js highlighting, put attributes on <code>
+       -- with language- prefix, and let highlight.js do the highlighting.
+       modify (\st -> st{ stHighlighting = True })
+       let (langClasses, otherClasses) = case classes' of
+             (lang:rest) -> (["language-" <> lang], rest)
+             []          -> ([], [])
+           codeAttrs = (id', langClasses ++ otherClasses, keyvals)
+       codeTag <- addAttrs opts codeAttrs $ H.code $ toHtml adjCode
+       return $ H.pre codeTag
+     else do
+       let highlighted = highlight (writerSyntaxMap opts)
+                           (if html5 then formatHtmlBlock else formatHtml4Block)
+                           (id'',classes',keyvals) adjCode
+           hlCode   = case writerHighlightMethod opts of
+                        Skylighting _ -> highlighted
+                        DefaultHighlighting -> highlighted
+                        _ -> Left ""
+       case hlCode of
+              Left msg -> do
+                unless (T.null msg) $
+                  report $ CouldNotHighlight msg
+                addAttrs opts (id',classes,keyvals)
+                  $ H.pre $ H.code $ toHtml adjCode
+              Right h -> modify (\st -> st{ stHighlighting = True }) >>
+                         -- we set writerIdentifierPrefix to "" since id'' already
+                         -- includes it:
+                         addAttrs opts{writerIdentifierPrefix = ""} (id'',[],keyvals) h
 blockToHtmlInner opts (BlockQuote blocks) = do
   -- in S5, treat list in blockquote specially
   -- if default is incremental, make it nonincremental;
@@ -1046,7 +1068,7 @@ blockToHtmlInner opts (OrderedList (startnum, numstyle, _) lst) = do
                                    numstyle']
                    else [])
   l <- ordList opts contents
-  return $ foldl' (!) l attribs
+  return $ L.foldl' (!) l attribs
 blockToHtmlInner opts (DefinitionList lst) = do
   contents <- mapM (\(term, defs) ->
                   do term' <- liftM H.dt $ inlineListToHtml opts term
@@ -1413,7 +1435,7 @@ inlineToHtml opts inline = do
                                = Just (t . H.u, cs)
                              | otherwise
                                = Just (t, c:cs)
-                            spanLikeTags = foldl' go Nothing
+                            spanLikeTags = L.foldl' go Nothing
                         in case spanLikeTags classes of
                             Just (tag, cs) -> do
                               h <- inlineListToHtml opts ils
@@ -1610,7 +1632,7 @@ inlineToHtml opts inline = do
                               Just "audio" -> mediaTag H5.audio "Audio"
                               Just _       -> (H5.embed, [])
                               _            -> imageTag
-                        return $ foldl' (!) tag $ attributes ++ specAttrs
+                        return $ L.foldl' (!) tag $ attributes ++ specAttrs
                         -- note:  null title included, as in Markdown.pl
     (Note contents) -> do
                         notes <- gets stNotes
@@ -1688,14 +1710,16 @@ blockListToNote opts ref blocks = do
       let kvs = [("role","doc-backlink") | html5]
       let backlink = Link ("",["footnote-back"],kvs)
                         [Str ref] ("#" <> "fnref" <> ref,"")
-      let blocks' =
-           case blocks of
-             (Para ils : rest) ->
-                Para (backlink : Str "." : Space : ils) : rest
-             (Plain ils : rest) ->
-                Plain (backlink : Str "." : Space : ils) : rest
-             _ -> Para [backlink , Str "."] : blocks
-      contents <- blockListToHtml opts blocks'
+      let addBacklinkInlines bs
+             | epubv == EPUB3 = bs
+             | otherwise =
+                 case bs of
+                   (Para ils : rest) ->
+                     Para (backlink : Str "." : Space : ils) : rest
+                   (Plain ils : rest) ->
+                     Plain (backlink : Str "." : Space : ils) : rest
+                   _ -> Para [backlink , Str "."] : blocks
+      contents <- blockListToHtml opts (addBacklinkInlines blocks)
       let noteItem = (if epubv == EPUB3
                          then H5.aside ! customAttribute "epub:type" "footnote" ! customAttribute "role" "doc-footnote"
                          else H.div) ! prefixedId opts ("fn" <> ref)
