@@ -1,10 +1,18 @@
+{-# LANGUAGE OverloadedStrings #-}
 module Tests.Writers.Docx (tests) where
 
-import Text.Pandoc
+import Codec.Archive.Zip (findEntryByPath, fromEntry, toArchive)
+import qualified Data.ByteString.Lazy as BL
+import Data.List (isInfixOf, isPrefixOf)
+import qualified Data.Map as M
+import Data.Text (Text)
+import qualified Data.Text as Text
+import qualified Data.Text.IO as T
 import Test.Tasty
-import Tests.Writers.OOXML
 import Test.Tasty.HUnit
-import Data.List (isPrefixOf)
+import Tests.Writers.OOXML
+import Text.Pandoc
+import Text.XML.Light (QName(QName), findAttr, findElements, parseXMLDoc)
 
 -- we add an extra check to make sure that we're not writing in the
 -- toplevel docx directory. We don't want to accidentally overwrite an
@@ -214,4 +222,74 @@ tests = [ testGroup "inlines"
             "docx/document-properties-short-desc.native"
             "docx/golden/document-properties-short-desc.docx"
           ]
+        , testGroup "reference docx"
+          [ testCase "no media directory override in content types" $ do
+              let opts = def{ writerReferenceDoc = Just "docx/inline_images.docx" }
+              txt <- T.readFile "docx/inline_formatting.native"
+              bs <- runIOorExplode $ do
+                mblang <- toLang (Just (Text.pack "en-US") :: Maybe Text)
+                maybe (return ()) setTranslations mblang
+                setVerbosity ERROR
+                readNative def txt >>= writeDocx opts
+              let archive = toArchive bs
+              entry <- case findEntryByPath "[Content_Types].xml" archive of
+                Nothing -> assertFailure "Missing [Content_Types].xml in output docx"
+                Just e -> return e
+              doc <- case parseXMLDoc (fromEntry entry) of
+                Nothing -> assertFailure "Failed to parse [Content_Types].xml"
+                Just d -> return d
+              let partNameAttr = QName "PartName" Nothing Nothing
+              let overrideName = QName "Override" Nothing Nothing
+              let overrides = findElements overrideName doc
+              let hasBadOverride =
+                    any (\el -> findAttr partNameAttr el == Just "/word/media/")
+                        overrides
+              assertBool "Found invalid /word/media/ Override in [Content_Types].xml"
+                (not hasBadOverride)
+          , testCase "language from reference docx is preserved" $ do
+              -- First, verify that the german-reference.docx actually has de-DE
+              refBs <- BL.readFile "docx/german-reference.docx"
+              let refArchive = toArchive refBs
+              refEntry <- case findEntryByPath "word/styles.xml" refArchive of
+                Nothing -> assertFailure "Missing word/styles.xml in german-reference.docx"
+                Just e -> return e
+              let refStylesXml = show (fromEntry refEntry)
+              let getLangLines = filter ("w:lang" `isInfixOf`) . lines
+              assertBool ("german-reference.docx w:lang line: " ++
+                  unlines (getLangLines refStylesXml))
+                (any ("de-DE" `isInfixOf`) (getLangLines refStylesXml))
+              -- Now test that using this reference preserves the language
+              let opts = def{ writerReferenceDoc = Just "docx/german-reference.docx" }
+              txt <- T.readFile "docx/inline_formatting.native"
+              bs <- runIOorExplode $ do
+                setVerbosity ERROR
+                readNative def txt >>= writeDocx opts
+              let archive = toArchive bs
+              entry <- case findEntryByPath "word/styles.xml" archive of
+                Nothing -> assertFailure "Missing word/styles.xml in output docx"
+                Just e -> return e
+              let stylesXml = show (fromEntry entry)
+              -- Find the w:lang line for debugging
+              -- Check that the styles.xml contains the German language
+              assertBool ("Language from reference docx not preserved. w:lang lines: " ++ unlines (getLangLines stylesXml))
+                (any ("de-DE" `isInfixOf`) (getLangLines stylesXml))
+          , testCase "language from metadata overrides reference docx" $ do
+              -- Use a reference docx with German language, but specify French in metadata
+              let opts = def{ writerReferenceDoc = Just "docx/german-reference.docx" }
+              bs <- runIOorExplode $ do
+                setVerbosity ERROR
+                -- Create a document with French language metadata
+                let doc = Pandoc (Meta $ M.fromList [("lang", MetaString "fr-FR")])
+                                 [Para [Str "Test"]]
+                writeDocx opts doc
+              let archive = toArchive bs
+              entry <- case findEntryByPath "word/styles.xml" archive of
+                Nothing -> assertFailure "Missing word/styles.xml in output docx"
+                Just e -> return e
+              let stylesXml = show (fromEntry entry)
+              -- Check that the styles.xml contains the French language (not German)
+              let getLangLines = filter ("w:lang" `isInfixOf`) . lines
+              assertBool "Language from metadata did not override reference docx (expected fr-FR)"
+                (any ("fr-FR" `isInfixOf`) (getLangLines stylesXml))
+          ]
         ]
