@@ -84,6 +84,11 @@ tests =
       doubleQuoted (spanWith ("", [], [("cite", "http://example.org")]) (str "examples"))
       =?> "<q cite=\"http://example.org\">examples</q>"
     ]
+  , testGroup "code"
+    [ "code should be rendered correctly" =:
+      plain (codeWith ("",[],[]) "Answer is 42") =?>
+      "<code>Answer is 42</code>"
+    ]
   , testGroup "sample"
     [ "sample should be rendered correctly" =:
       plain (codeWith ("",["sample"],[]) "Answer is 42") =?>
@@ -97,13 +102,13 @@ tests =
   , testGroup "sample with style"
     [ "samp should wrap highlighted code" =:
       codeWith ("",["sample","haskell"],[]) ">>="
-      =?> ("<samp><code class=\"sourceCode haskell\">" ++
+      =?> ("<samp><code class=\"sourceCode haskell sample\">" ++
           "<span class=\"op\">&gt;&gt;=</span></code></samp>")
     ]
   , testGroup "variable with style"
     [ "var should wrap highlighted code" =:
       codeWith ("",["haskell","variable"],[]) ">>="
-      =?> ("<var><code class=\"sourceCode haskell\">" ++
+      =?> ("<var><code class=\"sourceCode haskell variable\">" ++
           "<span class=\"op\">&gt;&gt;=</span></code></var>")
     ]
   , testGroup "footnotes"
@@ -188,7 +193,6 @@ tests =
           , "<p>A note inside a block quote.<a href=\"#fn2\" class=\"footnote-ref\" id=\"fnref2\"><sup>2</sup></a></p>"
           , "<p>A second paragraph.</p>"
           , "</blockquote>"
-          , "</div>"
           , "<div class=\"footnotes footnotes-end-of-section\">"
           , "<hr />"
           , "<ol>"
@@ -196,6 +200,7 @@ tests =
           , "<li id=\"fn2\"><p>The second note.<a href=\"#fnref2\" class=\"footnote-back\">↩︎</a></p></li>"
           , "</ol>"
           , "</div>"
+          , "</div>"
           , "<div class=\"section level2\">"
           , "<h2>Second section</h2>"
           , "<p>Some more text.</p>"
